import pytest
import pickle

from babelfish.country import Country
from babelfish.language import LANGUAGES, Language
from babelfish.script import Script


def test_languages():
    assert len(LANGUAGES) == 7874

def test_eq():
    assert Language("eng") == Language("eng")

def test_ne():
    assert Language("eng") != Language("fra")

def test_hasattr_alpha3():
    assert hasattr(Language("fra"), "alpha3")

def test_hasattr_alpha2():
    assert hasattr(Language("eng"), "alpha2")

def test_not_hasattr_alpha2():
    assert not hasattr(Language("bej"), "alpha2")

def test_bool_casting_is_true():
    assert bool(Language("eng"))

def test_bool_casting_und_is_false():
    assert not bool(Language("und"))

def test_wrong():
    with pytest.raises(ValueError):
        Language("xyz")

def test_wrong_as_unknown():
    assert Language("zzzz", unknown="und") == Language("und")

def test_eq_with_country():
    assert Language("eng", "US") == Language("eng", Country("US"))

def test_ne_with_country():
    assert Language("fra", "FR") != Language("fra", "CA")

def test_ne_with_country_and_none():
    assert Language("fra", "FR") != Language("fra")

def test_eq_with_script():
    assert Language("srp", script="Latn") == Language("srp", script=Script("Latn"))

def test_ne_with_script():
    assert Language("srp", script="Latn") != Language("srp", script="Cyrl")

def test_ne_with_script_and_none():
    assert Language("srp", script="Latn") != Language("srp")

def test_hash():
    assert hash(Language("fra")) == hash("fr")
    assert hash(Language("ace")) == hash("ace")
    assert hash(Language("por", "BR")) == hash("pt-BR")
    assert hash(Language("srp", script="Cyrl")) == hash("sr-Cyrl")
    assert hash(Language("eng", country="US", script="Latn")) == hash("en-US-Latn")

@pytest.mark.parametrize("language", [
    Language('fra'),
    Language('eng', 'US'),
    Language('srp', script='Latn'),
    Language('eng', 'US', 'Latn')
])
def test_pickle(language):
    assert pickle.loads(pickle.dumps(language)) == language

def test_str_ietf_format():
    assert str(Language("eng", "US", "Latn")) == "en-US-Latn"
    assert str(Language("fra", "FR")) == "fr-FR"
    assert str(Language("srp", script="Cyrl")) == "sr-Cyrl"
    assert str(Language("bel")) == "be"

def test_fromietf_with_country_and_script():
    assert Language.fromietf("fra-FR-Latn") == Language("fra", "FR", "Latn")

def test_fromietf_with_country_and_no_script():
    assert Language.fromietf("fr-FR") == Language("fra", "FR")

def test_fromietf_with_script_and_no_country():
    assert Language.fromietf("eng-Latn") == Language("eng", script="Latn")

def test_fromietf_wrong_language_raises_valueerror():
    with pytest.raises(ValueError):
        Language.fromietf("xyz-FR")

def test_fromietf_wrong_country_raises_valueerror():
    with pytest.raises(ValueError):
        Language.fromietf("eng-XY")

def test_fromietf_wrong_script_raises_valueerror():
    with pytest.raises(ValueError):
        Language.fromietf("fra-FR-Wxyz")




from babelfish.converters import LanguageReverseConverter
import pytest
from babelfish import language_converters
from babelfish.compat import resource_stream
from babelfish.exceptions import LanguageConvertError, LanguageReverseError
from babelfish.language import Language


def test_converter_alpha2():
    assert Language('eng').alpha2 == 'en'
    assert Language.fromalpha2('en') == Language('eng')
    assert Language.fromcode('en', 'alpha2') == Language('eng')
    with pytest.raises(LanguageReverseError):
        Language.fromalpha2('zz')
    with pytest.raises(LanguageConvertError):
        Language('aaa').alpha2
    assert len(language_converters['alpha2'].codes) == 184

def test_converter_alpha3b():
    assert Language('fra').alpha3b == 'fre'
    assert Language.fromalpha3b('fre') == Language('fra')
    assert Language.fromcode('fre', 'alpha3b') == Language('fra')
    with pytest.raises(LanguageReverseError):
        Language.fromalpha3b('zzz')
    with pytest.raises(LanguageConvertError):
        Language('aaa').alpha3b
    assert len(language_converters['alpha3b'].codes) == 418

def test_converter_alpha3t():
    assert Language('fra').alpha3t == 'fra'
    assert Language.fromalpha3t('fra') == Language('fra')
    assert Language.fromcode('fra', 'alpha3t') == Language('fra')
    with pytest.raises(LanguageReverseError):
        Language.fromalpha3t('zzz')
    with pytest.raises(LanguageConvertError):
        Language('aaa').alpha3t
    assert len(language_converters['alpha3t'].codes) == 418

def test_converter_name():
    assert Language('eng').name == 'English'
    assert Language.fromname('English') == Language('eng')
    assert Language.fromcode('English', 'name') == Language('eng')
    with pytest.raises(LanguageReverseError):
        Language.fromname('Zzzzzzzzz')
    assert len(language_converters['name'].codes) == 7874

def test_converter_scope():
    assert language_converters['scope'].codes == set(['I', 'S', 'M'])
    assert Language('eng').scope == 'individual'
    assert Language('und').scope == 'special'

def test_converter_type():
    assert language_converters['type'].codes == set(['A', 'C', 'E', 'H', 'L', 'S'])
    assert Language('eng').type == 'living'
    assert Language('und').type == 'special'

def test_converter_opensubtitles():
    assert Language('fra').opensubtitles == Language('fra').alpha3b
    assert Language('por', 'BR').opensubtitles == 'pob'
    assert Language('zho', 'TW').opensubtitles == 'zht'
    assert Language.fromopensubtitles('fre') == Language('fra')
    assert Language.fromopensubtitles('pob') == Language('por', 'BR')
    assert Language.fromopensubtitles('zht') == Language('zho', 'TW')
    assert Language.fromopensubtitles('pb') == Language('por', 'BR')
    # Montenegrin is not recognized as an ISO language (yet?) but for now it is
    # unofficially accepted as Serbian from Montenegro
    assert Language.fromopensubtitles('mne') == Language('srp', 'ME')
    assert Language.fromcode('pob', 'opensubtitles') == Language('por', 'BR')
    with pytest.raises(LanguageReverseError):
        Language.fromopensubtitles('zzz')
    with pytest.raises(LanguageConvertError):
        Language('aaa').opensubtitles
    assert len(language_converters['opensubtitles'].codes) == 608

    # test with all the LANGUAGES from the opensubtitles api
    # downloaded from: http://www.opensubtitles.org/addons/export_languages.php
    f = resource_stream('babelfish', 'data/opensubtitles_languages.txt')
    f.readline()
    for l in f:
        idlang, alpha2, _, upload_enabled, web_enabled = l.decode('utf-8').strip().split('\t')
        if not int(upload_enabled) and not int(web_enabled):
            # do not test LANGUAGES that are too esoteric / not widely available
            continue
        assert Language.fromopensubtitles(idlang).opensubtitles == idlang
        if alpha2:
            assert Language.fromopensubtitles(idlang) == Language.fromopensubtitles(alpha2)
    f.close()

def test_converter_opensubtitles_codes():
    for code in language_converters['opensubtitles'].from_opensubtitles.keys():
        assert code in language_converters['opensubtitles'].codes

def test_register_converter():
    class TestConverter(LanguageReverseConverter):
        def __init__(self):
            self.to_test = {'fra': 'test1', 'eng': 'test2'}
            self.from_test = {'test1': 'fra', 'test2': 'eng'}

        def convert(self, alpha3, country=None, script=None):
            if alpha3 not in self.to_test:
                raise LanguageConvertError(alpha3, country, script)
            return self.to_test[alpha3]

        def reverse(self, test):
            if test not in self.from_test:
                raise LanguageReverseError(test)
            return (self.from_test[test], None)
    language = Language('fra')
    assert not hasattr(language, 'test')
    language_converters['test'] = TestConverter()
    assert hasattr(language, 'test')
    assert 'test' in language_converters
    assert Language('fra').test == 'test1'
    assert Language.fromtest('test2').alpha3 == 'eng'
    del language_converters['test']
    assert 'test' not in language_converters
    with pytest.raises(KeyError):
        Language.fromtest('test1')
    with pytest.raises(AttributeError):
        Language('fra').test




# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from . import LanguageConverter
from ..exceptions import LanguageConvertError
from ..language import LANGUAGE_MATRIX


class LanguageTypeConverter(LanguageConverter):
    FULLNAME = {'A': 'ancient', 'C': 'constructed', 'E': 'extinct', 'H': 'historical', 'L': 'living', 'S': 'special'}
    SYMBOLS = {}
    for iso_language in LANGUAGE_MATRIX:
        SYMBOLS[iso_language.alpha3] = iso_language.type
    codes = set(SYMBOLS.values())

    def convert(self, alpha3, country=None, script=None):
        if self.SYMBOLS[alpha3] in self.FULLNAME:
            return self.FULLNAME[self.SYMBOLS[alpha3]]
        raise LanguageConvertError(alpha3, country, script)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from . import LanguageReverseConverter, CaseInsensitiveDict
from ..exceptions import LanguageReverseError
from ..language import language_converters


class OpenSubtitlesConverter(LanguageReverseConverter):
    def __init__(self):
        self.alpha3b_converter = language_converters['alpha3b']
        self.alpha2_converter = language_converters['alpha2']
        self.to_opensubtitles = {('por', 'BR'): 'pob', ('gre', None): 'ell', ('srp', None): 'scc', ('srp', 'ME'): 'mne', ('chi', 'TW'): 'zht'}
        self.from_opensubtitles = CaseInsensitiveDict({'pob': ('por', 'BR'), 'pb': ('por', 'BR'), 'ell': ('ell', None),
                                                       'scc': ('srp', None), 'mne': ('srp', 'ME'), 'zht': ('zho', 'TW')})
        self.codes = (self.alpha2_converter.codes | self.alpha3b_converter.codes | set(self.from_opensubtitles.keys()))

    def convert(self, alpha3, country=None, script=None):
        alpha3b = self.alpha3b_converter.convert(alpha3, country, script)
        if (alpha3b, country) in self.to_opensubtitles:
            return self.to_opensubtitles[(alpha3b, country)]
        return alpha3b

    def reverse(self, opensubtitles):
        if opensubtitles in self.from_opensubtitles:
            return self.from_opensubtitles[opensubtitles]
        for conv in [self.alpha3b_converter, self.alpha2_converter]:
            try:
                return conv.reverse(opensubtitles)
            except LanguageReverseError:
                pass
        raise LanguageReverseError(opensubtitles)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals


class Error(Exception):
    """Base class for all exceptions in babelfish"""
    pass


class LanguageError(Error, AttributeError):
    """Base class for all language exceptions in babelfish"""
    pass


class LanguageConvertError(LanguageError):
    """Exception raised by converters when :meth:`~babelfish.converters.LanguageConverter.convert` fails

    :param string alpha3: alpha3 code that failed conversion
    :param country: country code that failed conversion, if any
    :type country: string or None
    :param script: script code that failed conversion, if any
    :type script: string or None

    """
    def __init__(self, alpha3, country=None, script=None):
        self.alpha3 = alpha3
        self.country = country
        self.script = script

    def __str__(self):
        s = self.alpha3
        if self.country is not None:
            s += '-' + self.country
        if self.script is not None:
            s += '-' + self.script
        return s


class LanguageReverseError(LanguageError):
    """Exception raised by converters when :meth:`~babelfish.converters.LanguageReverseConverter.reverse` fails

    :param string code: code that failed reverse conversion

    """
    def __init__(self, code):
        self.code = code

    def __str__(self):
        return repr(self.code)


class CountryError(Error, AttributeError):
    """Base class for all country exceptions in babelfish"""
    pass


class CountryConvertError(CountryError):
    """Exception raised by converters when :meth:`~babelfish.converters.CountryConverter.convert` fails

    :param string alpha2: alpha2 code that failed conversion

    """
    def __init__(self, alpha2):
        self.alpha2 = alpha2

    def __str__(self):
        return self.alpha2


class CountryReverseError(CountryError):
    """Exception raised by converters when :meth:`~babelfish.converters.CountryReverseConverter.reverse` fails

    :param string code: code that failed reverse conversion

    """
    def __init__(self, code):
        self.code = code

    def __str__(self):
        return repr(self.code)



import pickle
import pytest
from babelfish.script import Script


def test_wrong_script():
    with pytest.raises(ValueError):
        Script("Qwerty")

def test_eq():
    assert Script("Latn") == Script("Latn")

def test_ne():
    assert Script("Cyrl") != Script("Latn")

def test_hash_eq():
    assert hash(Script("Hira")) == hash(Script("Hira"))

def test_pickle():
    assert pickle.loads(pickle.dumps(Script("Latn"))) == Script("Latn")




# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from __future__ import unicode_literals
from collections import namedtuple
from . import basestr
from .compat import resource_stream

#: Script code to script name mapping
SCRIPTS = {}

#: List of countries in the ISO-15924 as namedtuple of code, number, name, french_name, pva and date
SCRIPT_MATRIX = []

#: The namedtuple used in the :data:`SCRIPT_MATRIX`
IsoScript = namedtuple('IsoScript', ['code', 'number', 'name', 'french_name', 'pva', 'date'])

f = resource_stream('babelfish', 'data/iso15924-utf8-20131012.txt')
f.readline()
for l in f:
    l = l.decode('utf-8').strip()
    if not l or l.startswith('#'):
        continue
    script = IsoScript._make(l.split(';'))
    SCRIPT_MATRIX.append(script)
    SCRIPTS[script.code] = script.name
f.close()


class Script(object):
    """A human writing system

    A script is represented by a 4-letter code from the ISO-15924 standard

    :param string script: 4-letter ISO-15924 script code

    """
    def __init__(self, script):
        if script not in SCRIPTS:
            raise ValueError('%r is not a valid script' % script)

        #: ISO-15924 4-letter script code
        self.code = script

    @property
    def name(self):
        """English name of the script"""
        return SCRIPTS[self.code]

    def __getstate__(self):
        return self.code

    def __setstate__(self, state):
        self.code = state

    def __hash__(self):
        return hash(self.code)

    def __eq__(self, other):
        if isinstance(other, basestr):
            return self.code == other
        if not isinstance(other, Script):
            return False
        return self.code == other.code

    def __ne__(self, other):
        return not self == other

    def __repr__(self):
        return '<Script [%s]>' % self

    def __str__(self):
        return self.code



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
import sys

if sys.version_info[0] >= 3:
    basestr = str
else:
    basestr = basestring

from .converters import (LanguageConverter, LanguageReverseConverter, LanguageEquivalenceConverter, CountryConverter,
    CountryReverseConverter)
from .country import country_converters, COUNTRIES, COUNTRY_MATRIX, Country
from .exceptions import Error, LanguageConvertError, LanguageReverseError, CountryConvertError, CountryReverseError
from .language import language_converters, LANGUAGES, LANGUAGE_MATRIX, Language
from .script import SCRIPTS, SCRIPT_MATRIX, Script



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 the BabelFish authors. All rights reserved.
# Use of this source code is governed by the 3-clause BSD license
# that can be found in the LICENSE file.
#
from sys import version_info as _python

if _python >= (3, 9):
    # introduced in python 3.9
    from importlib.resources import files
else:
    from importlib_resources import files


if _python >= (3, 10):
    # .select() was introduced in 3.10
    from importlib.metadata import entry_points, EntryPoint as _EntryPoint
else:
    from importlib_metadata import entry_points, EntryPoint as _EntryPoint


def resource_stream(pkg, path):
    return files(pkg).joinpath(f'{path}').open('rb')


def iter_entry_points(group, **kwargs):
    return entry_points().select(group=group, **kwargs)


class EntryPoint(_EntryPoint):
    @staticmethod
    def parse(eps):
        return EntryPoint(*map(str.strip, eps.split('=')), None)

    def resolve(self):
        return self.load()



import pickle
import pytest
from babelfish.country import Country


def test_eq():
    assert Country("US") == Country("US")

def test_ne():
    assert Country("GB") != Country("US")

def test_hash_eq():
    assert hash(Country("US")) == hash("US")

def test_hasattr_name():
    assert hasattr(Country("US"), "name")

def test_hasattr_alpha2():
    assert hasattr(Country("US"), "alpha2")

def test_wrong():
    with pytest.raises(ValueError):
        Country("ZZ")

def test_pickle():
    assert pickle.loads(pickle.dumps(Country("GB"))) == Country("GB")
